@@ -26,7 +26,6 @@
 #include <stdio.h>
 #include <string.h>
 #include <glib-object.h>
-#include <libgnome/gnome-i18n.h>
 
 #include "gnome-vfs-util.h"
 
@@ -60,6 +59,8 @@ struct GnomeGdkPixbufAsyncHandle {
 typedef struct {
     gint width;
     gint height;
+    gint input_width;
+    gint input_height;
     gboolean preserve_aspect_ratio;
 } SizePrepareContext;
 
@@ -104,6 +105,9 @@ size_prepared_cb (GdkPixbufLoader *loader,
 
 	g_return_if_fail (width > 0 && height > 0);
 
+	info->input_width = width;
+	info->input_height = height;
+	
 	if (width < info->width && height < info->height) return;
 	
 	if (info->preserve_aspect_ratio && 
@@ -166,6 +170,9 @@ gnome_gdk_pixbuf_new_from_uri_at_scale (const char *uri,
     GnomeVFSFileSize bytes_read;
     GdkPixbufLoader *loader;
     GdkPixbuf *pixbuf;	
+    GdkPixbufAnimation *animation;
+    GdkPixbufAnimationIter *iter;
+    gboolean has_frame;
     SizePrepareContext info;
 
     g_return_val_if_fail (uri != NULL, NULL);
@@ -181,10 +188,14 @@ gnome_gdk_pixbuf_new_from_uri_at_scale (const char *uri,
     if (1 <= width || 1 <= height) {
         info.width = width;
         info.height = height;
+	info.input_width = info.input_height = 0;
         info.preserve_aspect_ratio = preserve_aspect_ratio;        
         g_signal_connect (loader, "size-prepared", G_CALLBACK (size_prepared_cb), &info);
     }
-    while (1) {
+
+    has_frame = FALSE;
+
+    while (!has_frame) {
 	result = gnome_vfs_read (handle,
 				 buffer,
 				 sizeof (buffer),
@@ -202,6 +213,15 @@ gnome_gdk_pixbuf_new_from_uri_at_scale (const char *uri,
 	    result = GNOME_VFS_ERROR_WRONG_FORMAT;
 	    break;
 	}
+
+	animation = gdk_pixbuf_loader_get_animation (loader);
+	if (animation) {
+		iter = gdk_pixbuf_animation_get_iter (animation, 0);
+		if (!gdk_pixbuf_animation_iter_on_currently_loading_frame (iter)) {
+			has_frame = TRUE;
+		}
+		g_object_unref (iter);
+	}
     }
 
     gdk_pixbuf_loader_close (loader, NULL);
@@ -217,6 +237,10 @@ gnome_gdk_pixbuf_new_from_uri_at_scale (const char *uri,
     pixbuf = gdk_pixbuf_loader_get_pixbuf (loader);
     if (pixbuf != NULL) {
 	g_object_ref (G_OBJECT (pixbuf));
+	g_object_set_data (G_OBJECT (pixbuf), "gnome-original-width",
+			   GINT_TO_POINTER (info.input_width));
+	g_object_set_data (G_OBJECT (pixbuf), "gnome-original-height",
+			   GINT_TO_POINTER (info.input_height));
     }
     g_object_unref (G_OBJECT (loader));
 
